from django.db import models
from django.conf import settings


class UserProfile(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="profile"
    )
    phone_number = models.CharField(
        max_length=15,
        null=True, blank=True
    )



"""
Django settings for test_app project.
Generated by 'django-admin startproject' using Django 3.1.7.
For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/
For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

from pathlib import Path
import environ

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


env = environ.Env()

environ.Env.read_env(
    BASE_DIR / "test_app" / ".env"
)


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "uhtgm(e2y3@5%0x!wy#re#fn+51h*ck88^ocm7d1=hx^@(&7$7"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    #
    "widget_tweaks",
    "django_twilio_2fa.apps.Twilio2faConfig",
    #
    "users.apps.UsersConfig",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "test_app.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "test_app.wsgi.application"


# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "test",
    }
}

# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = "/static/"

# DJANGO-TWILIO-2FA

TWILIO_2FA_ACCOUNT_SID = env("TWILIO_2FA_ACCOUNT_SID")
TWILIO_2FA_AUTH_TOKEN = env("TWILIO_2FA_AUTH_TOKEN")
TWILIO_2FA_SERVICE_ID = env("TWILIO_2FA_SERVICE_ID")

TWILIO_2FA_ALLOWED_METHODS = None

def twilio_2fa_register_cb(user, phone_number):
    from users.models import UserProfile

    if not hasattr(user, "profile"):
        user.profile = UserProfile.objects.create(
            user=user
        )

    user.profile.phone_number = phone_number
    user.profile.save()

    return True

TWILIO_2FA_REGISTER_CB = twilio_2fa_register_cb

def twilio_2fa_phone_number(user=None):
    if user and hasattr(user, "profile"):
        return user.profile.phone_number

    return None

TWILIO_2FA_PHONE_NUMBER_CB = twilio_2fa_phone_number

TWILIO_2FA_ALLOW_CHANGE = False



from django.conf import settings
from django.core.exceptions import ValidationError
import phonenumbers
from phonenumbers.phonenumberutil import NumberParseException
from twilio.rest import Client as TwilioClient
from twilio.base.exceptions import TwilioRestException
from .conf import *


__all__ = [
    "get_twilio_client", "verify_phone_number", "parse_phone_number",
]


default_region = get_setting(
    "PHONE_NUMBER_DEFAULT_REGION",
    default="US"
)


def get_twilio_client(**kwargs):
    if "account_sid" not in kwargs:
        kwargs["account_sid"] = get_setting("ACCOUNT_SID")
        kwargs["auth_token"] = get_setting("AUTH_TOKEN")

    args = [
        kwargs.pop("account_sid"),
        kwargs.pop("auth_token"),
    ]

    return TwilioClient(*args, **kwargs)


def parse_phone_number(phone_number):
    try:
        return phonenumbers.parse(phone_number, default_region)
    except NumberParseException as e:
        raise ValidationError(str(e))


def verify_phone_number(phone_number, do_lookup=False):
    allowed_country_codes = get_setting(
        "PHONE_NUMBER_ALLOWED_COUNTRIES",
        default=["US"]
    )

    disallowed_country_codes = get_setting(
        "PHONE_NUMBER_DISALLOWED_COUNTRIES",
        default=[]
    )

    do_lookup_setting = get_setting(
        "PHONE_NUMBER_CARRIER_LOOKUP",
        default=True
    )

    allowed_carrier_types = get_setting(
        "PHONE_NUMBER_ALLOWED_CARRIER_TYPES",
        default=["mobile"]
    )

    if do_lookup and not do_lookup_setting:
        do_lookup = False

    phone_number = parse_phone_number(phone_number)

    if not phonenumbers.is_valid_number(phone_number):
        raise ValidationError("Invalid phone number")

    if do_lookup:
        try:
            response = (get_twilio_client().lookups
                .phone_numbers(phone_number.national_number)
                .fetch(type=["carrier"])
            )
        except TwilioRestException as e:
            raise ValidationError("Unable to valid your phone number at this time. Please try again later.")

        country_code = response.country_code
        carrier_type = response.carrier.get("type", "")

        if carrier_type not in allowed_carrier_types:
            raise ValidationError(f"{carrier_type.title()} phone numbers are not allowed. "
                                  f"Must be a {' or '.join(allowed_carrier_types)} phone number.")

        if country_code not in allowed_country_codes or country_code in disallowed_country_codes:
            raise ValidationError(f"We do not allow phone numbers originating from {country_code}.")

    return True



from django.conf import settings


__all__ = [
    "SETTING_PREFIX", "get_setting_name", "get_setting"
]


SETTING_PREFIX = "TWILIO_2FA_"


def get_setting_name(name):
    return f"{SETTING_PREFIX}{name.upper()}"


def get_setting(name, default=None, callback_kwargs=None):
    if not name.upper().startswith(SETTING_PREFIX):
        name = get_setting_name(name)

    if not hasattr(settings, name):
        return default

    value = getattr(settings, name)

    must_be_callable = True if name.endswith("_CB") else False

    if callable(value):
        if not callback_kwargs:
            callback_kwargs = {}

        return value(**callback_kwargs)
    elif must_be_callable and not callable(value):
        raise ValueError(f"Setting {name} must be callable")

    return value
